import React, { createContext, useEffect, useState } from 'react';
import { TagInstanceInfo } from '../api';
import { Occurrence, Tag } from '../components/types/tag';

/**
 * Interface of an annotation context provider.
 */
interface IAnnotationContextProvider {
    /**
     * Tags managed by the context.
     */
    tags: TagInstanceInfo[] | null;

    /**
     * Sets new tags.
     * @param newTags An array of new tags.
     */
    setTags: (newTags: TagInstanceInfo[] | null) => void;

    /**
     * Adds occurrence to an annotation.
     * @param tag Tag whose occurrence should be added.
     */
    addOccurrence: (tag: Tag) => void;

    /**
     * Changes visibility of an annotation.
     * @param tag Tag whose visibility should be changed.
     */
    changeVisibility: (tag: Tag) => void;

    /**
     * Deletes an occurrence of an annotation.
     * @param occurrence Occurrence that should be deleted.
     */
    deleteOccurrence: (occurrence: Occurrence) => void;

    /**
     * Changes a position of an occurrence of an annotation.
     * @param occurrence Occurrence whose position should be changed.
     * @param newValue New value of the position.
     */
    changePosition: (occurrence: Occurrence, newValue: number) => void;

    /**
     * Changes a length of an occurrence of an annotation.
     * @param occurrence Occurrence whose length should be changed.
     * @param newValue New value of the length.
     */
    changeLength: (occurrence: Occurrence, newValue: number) => void;
}

/**
 * The annotation context that manages active annotations.
 */
export const AnnotationContext = createContext<IAnnotationContextProvider>({
    /**
     * Default tags.
     */
    tags: null,

    /**
     * Default implementation of setTags method.
     * @param v Array of new tags.
     */
    setTags: (v) => {
        return;
    },

    /**
     * Default implementation of addOccurrence method.
     * @param tag The tag with new occurrence.
     */
    addOccurrence: (tag: Tag) => {
        return;
    },

    /**
     * Default implementation of changeVisibility method.
     * @param tag The tag whose visibility should be changed.
     */
    changeVisibility: (tag: Tag) => {
        return;
    },

    /**
     * Default implementation of deleteOccurrence method.
     * @param occurrence Occurrence that should be deleted.
     */
    deleteOccurrence: (occurrence: Occurrence) => {
        return;
    },

    /**
     * Default implementation of changePosition method.
     * @param occurrence Occurrence whose position should be changed.
     * @param newValue A new position.
     */
    changePosition: (occurrence: Occurrence, newValue: number) => {
        return;
    },

    /**
     * Default implementation of changeLength method.
     * @param occurrence Occurrence whose length should be changed.
     * @param newValue A new length.
     */
    changeLength: (occurrence: Occurrence, newValue: number) => {
        return;
    },
});

/**
 * Provider of the annotation context.
 * @param props Children that should have access to the annotation context.
 * @returns Prepared html of the provider.
 */
const AnnotationProvider = (props: { children: React.ReactNode }) => {
    /**
     * Tags managed by the context.
     */
    const [tags, setTags] = useState<TagInstanceInfo[] | null>(null);

    /**
     * Default implementation of addOccurrence method.
     * @param tag The tag with new occurrence.
     */
    const addOccurrence = (tag: Tag) => {
        //TODO: Implement method (should use objects from server API)
    };

    /**
     * Changes visibility of an annotation.
     * @param tag Tag whose visibility should be changed.
     */
    const changeVisibility = (tag: Tag) => {
        //TODO: Implement method (should use objects from server API)
    };

    /**
     * Deletes an occurrence of an annotation.
     * @param occurrence Occurrence that should be deleted.
     */
    const deleteOccurrence = (occurrence: Occurrence) => {
        //TODO: Implement method (should use objects from server API)
    };

    /**
     * Changes a position of an occurrence of an annotation.
     * @param occurrence Occurrence whose position should be changed.
     * @param newValue New value of the position.
     */
    const changePosition = (occurrence: Occurrence, newValue: number) => {
        //TODO: Implement method (should use objects from server API)
    };

    /**
     * Changes a length of an occurrence of an annotation.
     * @param occurrence Occurrence whose length should be changed.
     * @param newValue New value of the length.
     */
    const changeLength = (occurrence: Occurrence, newValue: number) => {
        //TODO: Implement method (should use objects from server API)
    };

    /**
     * Initializes the context.
     */
    useEffect(() => {
        //TODO: Implement initialization with values from the server
    }, []);

    return (
        <AnnotationContext.Provider
            value={{
                tags,
                setTags,
                addOccurrence,
                changeVisibility,
                deleteOccurrence,
                changeLength,
                changePosition,
            }}
        >
            {props.children}
        </AnnotationContext.Provider>
    );
};

export default AnnotationProvider;
